<?php
namespace Ep\Gath;

use Ep\Cache\AbstractCacheFactory;
use Ep\Http\AbstractHttpClientFactory;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Psr\Container\ContainerInterface;

/**
 * Stvara novu instancu Gath APIja.
 *
 * @package Ep\Gath
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/gath
 */
class GathFactory implements FactoryInterface
{
    /**
     * Stvara API.
     *
     * @param ContainerInterface $container
     * @param string $requestedName
     * @param array|null $options
     *
     * @return Gath|object
     *
     * @throws \Psr\Container\ContainerExceptionInterface
     * @throws \Psr\Container\NotFoundExceptionInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, ?array $options = null)
    {
        $app = $container->get('config')['app'] ?? [];
        $config = $container->get('config')['gath'] ?? [];
        $cacheConfig = $config['cache'];

        if (!isset($config['base_uri']) || empty($config['base_uri'])) {
            throw new \RuntimeException("Polje 'base_uri' je obavezno.");
        }

        $httpConfig = [
            'base_uri' => $config['base_uri'],
            'verify' => false,
            'debug' => $config['debug'] ?? false,
            'timeout' => $config['timeout'] ?? 2,
        ];

        if (($config['enable_cache'] ?? true) && !($app['development'] ?? false)) {
            $cacheFactory = new AbstractCacheFactory();
            $cache = $cacheFactory($container, '', $cacheConfig );
            $httpConfig['cache'] = $cache;
        }

        $httpFactory = new AbstractHttpClientFactory();
        $http = $httpFactory($container, '', $httpConfig);

        $logger = $container->get('gath.logger');

        return new Gath($config, $http, $logger);
    }
}
