<?php
namespace Ep\Gath\Moto;

use Dflydev\FigCookies\SetCookie;
use Dflydev\FigCookies\SetCookies;
use Ep\Middleware\BaseUrl;
use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use Psr\Http\Server\MiddlewareInterface;
use Psr\Http\Server\RequestHandlerInterface;
use Laminas\Diactoros\Response\RedirectResponse;
use Laminas\Form\FormInterface;

/**
 * Class SelectMotorcycleHandler
 *
 * @package Gath\Moto\Middleware
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/gath-moto
 */
class SelectMotorcycleHandler implements MiddlewareInterface
{
    /** @var FormInterface */
    private $form;

    /**
     * SelectMotorcycleHandler constructor.
     *
     * @param FormInterface $form
     */
    public function __construct(FormInterface $form)
    {
        $this->form = $form;
    }

    /**
     * Process an incoming server request and return a response, optionally delegating
     * response creation to a handler.
     *
     * @param ServerRequestInterface $request
     * @param RequestHandlerInterface $handler
     *
     * @return ResponseInterface
     */
    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        $removeCookie = false;

        if ('POST' === $request->getMethod()) {
            $post = $request->getParsedBody();

            if (array_key_exists('select-motorcycle', $post)) {
                $this->form->setData($post);

                if ($this->form->isValid()) {
                    $data = $this->form->getData();
                    $motorcycleId = intval($data['motorcycleId']);
                    $path = sprintf('/%d', $motorcycleId);
                    $uri = $request->getUri();

                    $uri = $uri->withPath(sprintf(
                        '%s%s%s',
                        $request->getAttribute(BaseUrl::BASE_URL),
                        $path,
                        $uri->getPath()
                    ));

                    $response = new RedirectResponse($uri);

                    $setCookies = (SetCookies::fromResponse($response))
                        ->with(SetCookie::createRememberedForever('_moto_id', $motorcycleId)->withPath('/'));

                    $response = $setCookies->renderIntoSetCookieHeader($response);

                    return $response;
                }
            } else if (array_key_exists('reset-motorcycle', $post)) {
                $removeCookie = true;
                $cookies = $request->getCookieParams();

                if (isset($cookies['_moto_id'])) {
                    unset($cookies['_moto_id']);
                    $request = $request->withCookieParams($cookies);
                }
            }
        }

        $response = $handler->handle($request);

        if ($removeCookie) {
            $response = SetCookies::fromResponse($response)
                ->with(
                    SetCookie::createExpired('_moto_id')
                        ->withPath('/')
                )
                ->renderIntoSetCookieHeader($response);
        }

        return $response;
    }
}
