<?php
declare(strict_types=1);

namespace Ep\Gath\Plugin;

use WP_Term;
use WP_Term_Query;

/**
 * Class Attributes
 *
 * @package Ep\Gath\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/ep-gath
 */
final class Attributes extends AbstractGathPlugin
{
    private $taxonomies;

    public function attach(): void
    {
        add_filter('terms_pre_query', [$this, 'terms_pre_query'], 10, 2);
        add_filter('pre_transient_wc_attribute_taxonomies', [$this, 'get_attribute_taxonomies']);
        add_filter('attributes_for_category', [$this, 'get_attributes_for_category'], 10, 2);
    }

    /**
     * @return object[]
     */
    public function get_attribute_taxonomies($categoryId = null): array
    {
        if (preg_match('#/autodijelovi/$#', $_SERVER['REQUEST_URI'])) {
            return [];
        }

        $attributes = $this->gath->getAttributes($categoryId);
        
        foreach ($attributes as $attribute) {
            $taxonomies['id:' . $attribute['id']] = (object) [
                'attribute_id' => (string) $attribute['id'],
                'attribute_name' => $attribute['slug'],
                'attribute_label' => $attribute['naziv'],
                'attribute_type' => 'select',
                'attribute_orderby' => 'name',
                'attribute_public' => 0,
                'attribute_values' => $attribute['values'],
            ];
        }

        return $taxonomies ?? [];
    }

    private function sort_values(&$values)
    {
        
    }

    public function terms_pre_query(?array $terms, WP_Term_Query $query)
    {
        $q = &$query->query_vars;
        $taxonomy = is_array($q['taxonomy']) ? $q['taxonomy'][0] : $q['taxonomy'];

        if (!is_string($taxonomy) || empty($taxonomy) || !preg_match('/^pa_(.+)$/', $taxonomy, $matches)) {
            return $terms;
        }

        $taxonomy = $matches[1];

        if ($q['parent'] !== '') {

        } else if (!empty($q['include'])) {

        } else if (!empty($q['slug'])) {

        } else {
            $taxonomies = $this->get_attribute_taxonomies();
            foreach ($taxonomies as $tax) {
                if ($tax->attribute_name === $taxonomy) {
                    foreach ($tax->attribute_values as $value) {
                        $terms[] = $this->load_term($tax, $value);
                    }

                    return $terms ?? [];
                }
            }
        }

        $fields = $q['fields'] ?: 'all';

        if ($fields === 'count') {
            return count($data);
        }

        foreach ($data ?? [] as $row) {
            $row = $this->prepareRow($row);

            switch ($fields) {
                case 'id=>parent':
                    $terms[$row['term_id']] = $row['parent'];
                    break;
                case 'all':
                case 'all_with_object_id':
                default:
                    $terms[] = new WP_Term((object) $row);
            }
        }

        return $terms ?? [];
    }
    
    /**
     * @param $attributes
     * @param int $categoryId
     * @return object[]
     * @noinspection PhpUnusedParameterInspection
     */
    public function get_attributes_for_category($attributes, int $categoryId): array
    {
        $category_ids = get_term_children($categoryId, 'product_cat');
        $category_ids[] = $categoryId;

        $category_ids = array_map(function ($id) {
            return intval($id - Products::TERM_OFFSET_CATEGORY);
        }, $category_ids);

        return $this->get_attribute_taxonomies($category_ids);
    }

    private function load_term(object $taxonomy, $value): WP_Term
    {
        $valueIndex = array_search($value, $taxonomy->attribute_values);
        $term_id = intval(intval(sprintf('%03d%03d', $taxonomy->attribute_id, $valueIndex)) + Products::TERM_OFFSET_ATTRIBUTE);
        $tax = 'pa_' . $taxonomy->attribute_name;

        return new WP_Term((object) [
            'term_id' => $term_id,
            'name' => $value,
            'slug' => sanitize_title($value),
            'taxonomy' => $tax,
            'description' => '',
            'parent' => 0,
        ]);
    }
}
