<?php
declare(strict_types=1);

namespace Ep\Gath\Plugin;

/**
 * Class Wishlist
 *
 * @package Ep\Gath\Plugin
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/ep-gath
 */
final class Wishlist extends AbstractGathPlugin
{
    public function attach(): void
    {
        add_filter('query', [$this, 'query']);
    }

    public function query (string $query): string
    {
        if (preg_match('/`cnt`.+FROM `wp_yith_wcwl`.+l\.`user_id` = (\d+) GROUP BY i\.prod_id, l\.ID$/s', $query, $matches)) {

            $productIds = $this->getWishlistProductIds();

            if (count($productIds) > 0) {
                $first = array_shift($productIds);
                $query = "SELECT $first AS cnt";

                foreach ($productIds as $productId) {
                    $query .= " UNION ALL $productId ";
                }
            }
        }

        return $query;
    }

    private function getUsername(): ?string
    {
        $user = wp_get_current_user();
        return $user ? $user->user_login : null;
    }

    private function getWishlistProductIds(): array
    {
        $username = $this->getUsername();
        if (!$username) return [];

        $results = $this->gath->getWishlistProducts($username);

        return array_map(
            fn (array $result) => $result['DataLink'],
            $results['Results'] ?? [],
        );
    }
}